package com.gremwell.jsersearch;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.ObjectInputStream;

import com.thoughtworks.xstream.XStream;

public class Main {

	/**
	 * @param args
	 */
	public static void main(String[] args) {
		if (args.length != 1) {
			usage();
		}

		File f = new File(args[0]);

		for (int offset = 0; offset < f.length() - 2; offset++) {
			FileInputStream f_in = openPositionedStream(f, offset);

			ObjectInputStream obj_in = openObjectStream(f_in, offset);
			if (obj_in == null)
				continue;

			System.out.println("Found objectStream at offset " + offset + ", dumping ...");
			try {
				dumpObjectStream(obj_in);
			} catch (Throwable t) {
				System.out.println("Caught exception while dumping " + t);
			}
			System.out.println("End of dump (from offset " + offset + ")");
		}
	}

	private static void dumpObjectStream(ObjectInputStream obj_in)
			throws ClassNotFoundException {
		XStream xstream = new XStream();

		for (int j = 0;; j++) {
			Object o1;
			try {
				o1 = obj_in.readObject();
			} catch (IOException e) {
				return;
			}

			System.out.println("#" + j + ": class " + o1.getClass());
			System.out.println("#" + j + ": XML dump: " + xstream.toXML(o1));
		}
	}

	private static ObjectInputStream openObjectStream(FileInputStream f_in,
			long offset) {
		try {
			ObjectInputStream obj_in = new ObjectInputStream(f_in);
			return obj_in;
		} catch (IOException e) {
			if (!e.toString()
					.startsWith(
							"java.io.StreamCorruptedException: invalid stream header: "))
				System.out.println("Offset " + offset + " exception "
						+ e.toString());
			return null;
		}
	}

	private static FileInputStream openPositionedStream(File f, int offset) {
		try {
			FileInputStream f_in = new FileInputStream(f);
			f_in.skip(offset);
			return f_in;
		} catch (IOException e) {
			throw new RuntimeException("open('" + f + "')/seek(" + offset
					+ ") failed", e);
		}
	}

	private static void usage() {
		System.err.println("Usage: jsersearch FILE");
		System.exit(1);
	}
}
